@@ -5,23 +5,50 @@ import itertools
 import copy
 import functools
 import random
-from collections.abc import Mapping, Iterable
+import warnings
+from collections.abc import Container, Iterable, Mapping
+from typing import Callable, Union
 
 import jaraco.text
 
 
+_Matchable = Union[Callable, Container, Iterable, re.Pattern]
+
+
+def _dispatch(obj: _Matchable) -> Callable:
+    # can't rely on singledispatch for Union[Container, Iterable]
+    # due to ambiguity
+    # (https://peps.python.org/pep-0443/#abstract-base-classes).
+    if isinstance(obj, re.Pattern):
+        return obj.fullmatch
+    if not isinstance(obj, Callable):  # type: ignore
+        if not isinstance(obj, Container):
+            obj = set(obj)  # type: ignore
+        obj = obj.__contains__
+    return obj  # type: ignore
+
+
 class Projection(collections.abc.Mapping):
     """
     Project a set of keys over a mapping
 
     >>> sample = {'a': 1, 'b': 2, 'c': 3}
     >>> prj = Projection(['a', 'c', 'd'], sample)
-    >>> prj == {'a': 1, 'c': 3}
+    >>> dict(prj)
+    {'a': 1, 'c': 3}
+
+    Projection also accepts an iterable or callable or pattern.
+
+    >>> iter_prj = Projection(iter('acd'), sample)
+    >>> call_prj = Projection(lambda k: ord(k) in (97, 99, 100), sample)
+    >>> pat_prj = Projection(re.compile(r'[acd]'), sample)
+    >>> prj == iter_prj == call_prj == pat_prj
     True
 
     Keys should only appear if they were specified and exist in the space.
+    Order is retained.
 
-    >>> sorted(list(prj.keys()))
+    >>> list(prj)
     ['a', 'c']
 
     Attempting to access a key not in the projection
@@ -36,38 +63,56 @@ class Projection(collections.abc.Mapping):
 
     >>> target = {'a': 2, 'b': 2}
     >>> target.update(prj)
-    >>> target == {'a': 1, 'b': 2, 'c': 3}
-    True
+    >>> target
+    {'a': 1, 'b': 2, 'c': 3}
 
-    Also note that Projection keeps a reference to the original dict, so
-    if you modify the original dict, that could modify the Projection.
+    Projection keeps a reference to the original dict, so
+    modifying the original dict may modify the Projection.
 
     >>> del sample['a']
     >>> dict(prj)
     {'c': 3}
     """
 
-    def __init__(self, keys: Iterable, space: Mapping):
-        self._keys = set(keys)
+    def __init__(self, keys: _Matchable, space: Mapping):
+        self._match = _dispatch(keys)
         self._space = space
 
     def __getitem__(self, key):
-        if key not in self._keys:
+        if not self._match(key):
             raise KeyError(key)
         return self._space[key]
 
     def _keys_resolved(self):
-        return self._keys.intersection(self._space)
+        return filter(self._match, self._space)
 
     def __iter__(self):
-        return iter(self._keys_resolved())
+        return self._keys_resolved()
 
     def __len__(self):
-        return len(self._keys_resolved())
+        return len(tuple(self._keys_resolved()))
+
+
+class Mask(Projection):
+    """
+    The inverse of a projection, masking out keys.
+
+    >>> sample = {'a': 1, 'b': 2, 'c': 3}
+    >>> msk = Mask(['a', 'c', 'd'], sample)
+    >>> dict(msk)
+    {'b': 2}
+    """
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # self._match = compose(operator.not_, self._match)
+        self._match = lambda key, orig=self._match: not orig(key)
 
 
 class DictFilter(Projection):
     """
+    *Deprecated*
+
     Takes a dict and simulates a sub-dict based on a pattern.
 
     >>> sample = dict(a=1, b=2, c=3, d=4, ef=5)
@@ -89,11 +134,11 @@ class DictFilter(Projection):
     Pattern is useful for excluding keys with a prefix.
 
     >>> filtered = DictFilter(sample, include_pattern=r'(?![ace])')
-    >>> dict(filtered) == dict(b=2, d=4)
+    >>> filtered == dict(b=2, d=4)
     True
 
-    Also note that DictFilter keeps a reference to the original dict, so
-    if you modify the original dict, that could modify the filtered dict.
+    DictFilter keeps a reference to the original dict, so
+    modifying the original dict may modify the filtered dict.
 
     >>> del sample['d']
     >>> dict(filtered)
@@ -101,12 +146,12 @@ class DictFilter(Projection):
     """
 
     def __init__(self, dict, *, include_pattern):
-        self._space = dict
-        self._include_pattern = re.compile(include_pattern)
-
-    @property
-    def _keys(self):
-        return set(filter(self._include_pattern.match, self._space))
+        warnings.warn(
+            "DictFilter is deprecated. Pass re.Pattern to Projection instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        super().__init__(re.compile(include_pattern).match, dict)
 
 
 def dict_map(function, dictionary):
@@ -133,7 +178,7 @@ class RangeMap(dict):
     One may supply keyword parameters to be passed to the sort function used
     to sort keys (i.e. key, reverse) as sort_params.
 
-    Let's create a map that maps 1-3 -> 'a', 4-6 -> 'b'
+    Create a map that maps 1-3 -> 'a', 4-6 -> 'b'
 
     >>> r = RangeMap({3: 'a', 6: 'b'})  # boy, that was easy
     >>> r[1], r[2], r[3], r[4], r[5], r[6]
@@ -145,7 +190,7 @@ class RangeMap(dict):
     >>> r[4.5]
     'b'
 
-    But you'll notice that the way rangemap is defined, it must be open-ended
+    Notice that the way rangemap is defined, it must be open-ended
     on one side.
 
     >>> r[0]
@@ -263,7 +308,7 @@ def __identity(x):
 
 def sorted_items(d, key=__identity, reverse=False):
     """
-    Return the items of the dictionary sorted by the keys
+    Return the items of the dictionary sorted by the keys.
 
     >>> sample = dict(foo=20, bar=42, baz=10)
     >>> tuple(sorted_items(sample))
